import { GraphQLError } from 'graphql/error';
import { GraphQLObjectType, GraphQLFieldResolver } from 'graphql/type/definition';
import { GraphQLSchema } from 'graphql/type/schema';
import { DocumentNode, OperationDefinitionNode, FragmentDefinitionNode } from 'graphql/language/ast';
import { ExecutionResult } from 'graphql/execution/execute';
import Maybe from 'graphql/tsutils/Maybe';
export declare type MaybePromise<T> = Promise<T> | T;
export declare type ExecutionArgs = {
    schema: GraphQLSchema;
    document: DocumentNode;
    rootValue?: any;
    contextValue?: any;
    variableValues?: Maybe<{
        [key: string]: any;
    }>;
    operationName?: Maybe<string>;
    fieldResolver?: Maybe<GraphQLFieldResolver<any, any>>;
    enableDefer?: boolean;
};
declare type FieldValue = Record<string, {}> | Array<any> | string | number | boolean | null;
declare type PatchBundle = Promise<{
    patch: ExecutionPatchResult;
    dependentPatches?: PatchBundle[];
}>;
export declare type ExecutionContext = {
    schema: GraphQLSchema;
    fragments: Record<string, FragmentDefinitionNode>;
    rootValue: {};
    contextValue: {};
    operation: OperationDefinitionNode;
    variableValues: {
        [variable: string]: {};
    };
    fieldResolver: GraphQLFieldResolver<any, any>;
    errors: GraphQLError[];
    enableDefer?: boolean;
    patchDispatcher?: PatchDispatcher;
    deferredDependents?: Record<string, {
        patches: PatchBundle[];
        errors: GraphQLError[];
    }>;
};
export declare function buildExecutionContext(schema: GraphQLSchema, document: DocumentNode, rootValue: {}, contextValue: {}, rawVariableValues: Record<string, {}> | null, operationName: string | null, fieldResolver: GraphQLFieldResolver<any, any> | null, enableDefer?: boolean): GraphQLError[] | ExecutionContext;
export interface ExecutionPatchResult {
    data?: FieldValue;
    errors?: ReadonlyArray<GraphQLError>;
    path: ReadonlyArray<string | number>;
}
export interface DeferredExecutionResult {
    initialResult: ExecutionResult;
    deferredPatches: AsyncIterable<ExecutionPatchResult>;
}
export declare function isDeferredExecutionResult(result: any): result is DeferredExecutionResult;
declare class PatchDispatcher {
    private resolvers;
    private resultPromises;
    dispatch(patch: PatchBundle): void;
    getAsyncIterable(): AsyncIterable<ExecutionPatchResult>;
}
export declare function execute(ExecutionArgs: ExecutionArgs, ..._: any[]): MaybePromise<ExecutionResult | DeferredExecutionResult>;
export declare function execute(schema: GraphQLSchema, document: DocumentNode, rootValue?: {}, contextValue?: {}, variableValues?: {
    [variable: string]: {};
}, operationName?: string, fieldResolver?: GraphQLFieldResolver<any, any>, enableDefer?: boolean): MaybePromise<ExecutionResult | DeferredExecutionResult>;
export declare function getOperationRootType(schema: GraphQLSchema, operation: OperationDefinitionNode): GraphQLObjectType;
export {};
//# sourceMappingURL=execute.d.ts.map